<?php
/**
 * "Get Resolver Links" AJAX handler
 *
 * PHP version 7
 *
 * Copyright (C) Villanova University 2018.
 * Copyright (C) Leipzig University Library 2019.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  AJAX
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   Graham Seaman <Graham.Seaman@rhul.ac.uk>
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @author   Frank Morgner <morgnerf@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace fid\AjaxHandler;

use VuFind\Resolver\Connection;
use VuFind\Resolver\Driver\PluginManager as ResolverManager;
use VuFind\Session\Settings as SessionSettings;
use Zend\Config\Config;
use Zend\Mvc\Controller\Plugin\Params;
use Zend\View\Renderer\RendererInterface;

/**
 * "Get Resolver Links" AJAX handler
 *
 * Fetch Links from resolver given an OpenURL and format as HTML
 * and output the HTML content in JSON object.
 *
 * @category VuFind
 * @package  AJAX
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   Graham Seaman <Graham.Seaman@rhul.ac.uk>
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @author   Frank Morgner <morgnerf@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
class GetResolverLinks extends \VuFind\AjaxHandler\GetResolverLinks
{
    /**
     * Resolver configuration
     *
     * @var Config
     */
    protected $resolverConfig;

    /**
     * Constructor
     *
     * @param SessionSettings   $ss       Session settings
     * @param ResolverManager   $pm       Resolver driver plugin manager
     * @param RendererInterface $renderer View renderer
     * @param Config            $config   Top-level VuFind configuration (config.ini)
     * @param Config            $resolver Top-level VuFind configuration (Resolver.ini)
     */
    public function __construct(
        SessionSettings $ss,
        ResolverManager $pm,
        RendererInterface $renderer,
        Config $config,
        Config $resolver
    ) {
        parent::__construct($ss, $pm, $renderer, $config);
        $this->resolverConfig = $resolver;
    }

    /**
     * Handle a request.
     *
     * Method can handle multiple link resolvers. VuFind standard configuration
     * of [OpenUrl] config.ini is replaced by finc defined Resolver.ini. Therefore
     * parameter $config is currently out of use.
     *
     * @param Params $params Parameter helper from controller
     *
     * @return array [response data, HTTP status code]
     */
    public function handleRequest(Params $params)
    {
        $this->disableSessionWrites();  // avoid session write timing bug
        $openUrl = $params()->fromQuery('openurl', '');
        $requestedResolver = $params()->fromQuery('resolvertype', '');
        $searchClassId = $params()->fromQuery('searchClassId', '');
        $recordId = $params->fromQuery('recordId', '');

        //$config = $this->getConfig('Resolver');
        $resolvers = explode(',', $this->resolverConfig->General->active_resolvers);

        if (in_array($requestedResolver, $resolvers)
            && isset($this->resolverConfig->$requestedResolver)
        ) {
            $resolverType = $this->resolverConfig->$requestedResolver->resolver ?? 'generic';
            if (!$this->pluginManager->has($resolverType)) {
                return $this->formatResponse(
                    $this->translate("Could not load driver for $resolverType"),
                    self::STATUS_HTTP_ERROR
                );
            }

            $resolver = new Connection($this->pluginManager->get($resolverType));
            if (isset($this->resolverConfig->$requestedResolver->resolver_cache)) {
                $resolver->enableCache(
                    $this->resolverConfig->$requestedResolver->resolver_cache
                );
            }
            $result = $resolver->fetchLinks($openUrl);

            if (isset($result['messages'])) {
                $messages =  array_unique($result['messages']);
                unset($result['messages']);
            } else {
                $messages = [];
            }

            if (isset($result['stateOfResults'])) {
                $stateOfResults = $result['stateOfResults'];
                unset($result['stateOfResults']);
            } else {
                $stateOfResults = [];
            }

            // Sort the returned links into categories based on service type:
            $electronic = $print = $services = [];
            foreach ($result as $link) {
                switch ($link['service_type'] ?? '') {
                case 'getHolding':
                    $print[] = $link;
                    break;
                case 'getWebService':
                    $services[] = $link;
                    break;
                case 'getDOI':
                    // Special case -- modify DOI text for special display:
                    $link['title'] = $this->translate('Get full text');
                    $link['coverage'] = '';
                    break;
                case 'getFullTxt':
                default:
                    $electronic[] = $link;
                    break;
                }
            }

            // Get the OpenURL base:
            if (isset($this->resolverConfig->$requestedResolver)
                && isset($this->resolverConfig->$requestedResolver->url)
            ) {
                // Trim off any parameters (for legacy compatibility -- default config
                // used to include extraneous parameters):
                [$base] = explode(
                    '?',
                    $this->resolverConfig->$requestedResolver->url
                );
            } else {
                $base = false;
            }

            $moreOptionsLink = $resolver->supportsMoreOptionsLink()
                ? $resolver->getResolverUrl($openUrl) : '';

            // Render the links using the view:
            $view = [
                'openUrlBase' => $base, 'openUrl' => $openUrl,
                'electronic' => $electronic, 'unknown' => $unknown, 'services' => $services,
                'searchClassId' => $searchClassId, 'resolver' => $requestedResolver,
                'moreOptionsLink' => $moreOptionsLink
            ];
            $viewPrint = [
                'print' => $print,
                'messages' => $messages
            ];
            $viewButton = [
                'showOrderButton' => $stateOfResults,
                'recordId' => $recordId
            ];
        }

        $html = $this->renderer->render("ajax/resolverLinks.phtml", $view);
        $print = $this->renderer->render("ajax/resolverLinks-print.phtml", $viewPrint);
        $button = $this->renderer->render("ajax/resolverLinks-button.phtml", $viewButton);

        // output HTML encoded in JSON object
        return $this->formatResponse(compact('html', 'print', 'button'));
    }
}
