<?php
/**
 * Copyright (C) 2019 Leipzig University Library
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @author  Sebastian Kehr <kehr@ub.uni-leipzig.de>
 * @license http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 */
namespace fid\Service\DataTransferObject;

namespace fid\Service\DataTransferObject;

use Symfony\Component\Serializer\Annotation\Groups;

class Order
{
    /**
     * @var      string|null
     * @Groups({
     *     "order:creation:response",
     *     "order:list:response",
     *     "order:update:request",
     *     "user:details:response",
     *     "user:creation:response",
     *     "user:update:response"
     * })
     */
    protected $id;

    /**
     * @var      string
     * @Groups({
     *     "order:creation:request",
     *     "order:creation:response",
     *     "order:update:request",
     *     "order:list:response",
     *     "user:details:response",
     *     "user:creation:response",
     *     "user:update:response"
     * })
     */
    protected $type = '';

    /**
     * @var      string
     * @Groups({
     *     "order:creation:request",
     *     "order:creation:response",
     *     "order:list:response",
     *     "order:update:request",
     *     "user:details:response",
     *     "user:creation:response",
     *     "user:update:response"
     * })
     */
    protected $label = '';

    /**
     * @var      User|null
     * @Groups({
     *     "order:creation:request",
     *     "order:creation:response",
     *     "order:list:response",
     *     "order:update:request",
     *     "user:details:response",
     *     "user:creation:response",
     *     "user:update:response"
     * })
     */
    protected $user;

    /**
     * @var      array
     * @Groups({
     *     "order:creation:request",
     *     "order:creation:response",
     *     "order:update:request",
     *     "order:list:response",
     *     "user:details:response",
     *     "user:creation:response",
     *     "user:update:response"
     * })
     */
    protected $data = [];

    /**
     * @var      \DateTime
     * @Groups({
     *     "order:creation:response",
     *     "order:list:response",
     *     "order:update:request",
     *     "user:details:response",
     *     "user:creation:response",
     *     "user:update:response"
     * })
     */
    protected $createdAt;

    public const DATETIME_FORMAT = 'Y-m-d H:i:s';

    /**
     * @return string|null
     */
    public function getId(): ?string
    {
        return $this->id;
    }

    /**
     * @param string|null $id
     */
    public function setId(?string $id): void
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getLabel(): string
    {
        return $this->label;
    }

    /**
     * @param string $label
     */
    public function setLabel(string $label): void
    {
        $this->label = $label;
    }

    /**
     * @return string
     */
    public function getType(): string
    {
        return $this->type;
    }

    /**
     * @param string $type
     */
    public function setType(string $type): void
    {
        $this->type = $type;
    }

    /**
     * @return User|null
     */
    public function getUser(): ?User
    {
        return $this->user;
    }

    /**
     * @param User|null $user
     */
    public function setUser(?User $user): void
    {
        $this->user = $user;
    }

    /**
     * @return array
     */
    public function getData(): array
    {
        return $this->data;
    }

    /**
     * @param array $data
     */
    public function setData(array $data): void
    {
        $this->data = $data;
    }

    /**
     * @return \DateTime
     */
    public function getCreatedAt(): \DateTime
    {
        return $this->createdAt;
    }

    /**
     * @param \DateTime $createdAt
     */
    public function setCreatedAt(\DateTime $createdAt): void
    {
        $this->createdAt = $createdAt;
    }

    public function __toString()
    {
        return $this->getLabel();
    }

    public function getDueDate(): ? \DateTime
    {
        return (!empty($this->data['dueDate']))
            ? \DateTime::createFromFormat(self::DATETIME_FORMAT, $this->data['dueDate'])
            : null;
    }

    /**
     * @param \DateTime $dueDate
     */
    public function setDueDate(\DateTime $date)
    {
        $data = $this->getData();
        $data['dueDate'] = $date->format(self::DATETIME_FORMAT);
        $this->setData($data);
    }

    public function displayDueDate(): String
    {
        return $this->getDueDate() ? $this->getDueDate()->format('d.m.Y') : '';
    }

    public function isOverdue() : bool
    {
        if ($duedate = $this->getDueDate()) {
            $now = new \DateTime();
            return $now > $duedate ? true : false;
        } else {
            return false;
        }
    }

    /**
     * @return string
     */
    public function getStatus(): ? string
    {
        return $this->data['status'] ?? null;
    }

    /**
     * @param string $status
     */
    public function setStatus(?string $status): void
    {
        if ($status != null) {
            $this->data['status'] = $status;
        }
    }

    public function getRecordId()
    {
        return $this->data['record']['id'] ?? null;
    }
}
